"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNodeDir = exports.handler = exports.help = exports.commandNames = exports.shorthands = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const fetch_1 = __importDefault(require("@pnpm/fetch"));
const execa_1 = __importDefault(require("execa"));
const path_name_1 = __importDefault(require("path-name"));
const render_help_1 = __importDefault(require("render-help"));
const load_json_file_1 = __importDefault(require("load-json-file"));
const write_json_file_1 = __importDefault(require("write-json-file"));
const rcOptionsTypes = () => ({});
exports.rcOptionsTypes = rcOptionsTypes;
const cliOptionsTypes = () => ({});
exports.cliOptionsTypes = cliOptionsTypes;
exports.shorthands = {};
exports.commandNames = ['node'];
function help() {
    return render_help_1.default({
        description: 'Run Node.js',
        descriptionLists: [],
        url: cli_utils_1.docsUrl('node'),
        usages: ['pnpm node'],
    });
}
exports.help = help;
async function handler(opts) {
    const nodeDir = await getNodeDir(opts.pnpmHomeDir, opts.useNodeVersion);
    const { exitCode } = await execa_1.default('node', opts.argv.original.slice(1), {
        env: {
            [path_name_1.default]: `${path_1.default.join(nodeDir, 'node_modules/.bin')}${path_1.default.delimiter}${process.env[path_name_1.default]}`,
        },
        stdout: 'inherit',
        stdin: 'inherit',
    });
    return { exitCode };
}
exports.handler = handler;
async function getNodeDir(pnpmHomeDir, nodeVersion) {
    var _a;
    const nodesDir = path_1.default.join(pnpmHomeDir, 'nodes');
    let wantedNodeVersion = nodeVersion !== null && nodeVersion !== void 0 ? nodeVersion : (_a = (await readNodeVersionsManifest(nodesDir))) === null || _a === void 0 ? void 0 : _a.default;
    await fs_1.default.promises.mkdir(nodesDir, { recursive: true });
    fs_1.default.writeFileSync(path_1.default.join(nodesDir, 'pnpm-workspace.yaml'), '', 'utf8');
    if (wantedNodeVersion == null) {
        const response = await fetch_1.default('https://registry.npmjs.org/node');
        wantedNodeVersion = (await response.json())['dist-tags'].lts;
        if (wantedNodeVersion == null) {
            throw new Error('Could not resolve LTS version of Node.js');
        }
        await write_json_file_1.default(path_1.default.join(nodesDir, 'versions.json'), {
            default: wantedNodeVersion,
        });
    }
    const versionDir = path_1.default.join(nodesDir, wantedNodeVersion);
    if (!fs_1.default.existsSync(versionDir)) {
        await installNode(wantedNodeVersion, versionDir);
    }
    return versionDir;
}
exports.getNodeDir = getNodeDir;
async function installNode(wantedNodeVersion, versionDir) {
    await fs_1.default.promises.mkdir(versionDir, { recursive: true });
    await write_json_file_1.default(path_1.default.join(versionDir, 'package.json'), {});
    const { exitCode } = await execa_1.default('pnpm', ['add', '--use-stderr', `${getNodePkgName()}@${wantedNodeVersion}`], {
        cwd: versionDir,
        stdout: 'inherit',
    });
    if (exitCode !== 0) {
        throw new Error(`Couldn't install Node.js ${wantedNodeVersion}`);
    }
}
function getNodePkgName() {
    const platform = process.platform === 'win32' ? 'win' : process.platform;
    const arch = platform === 'win' && process.arch === 'ia32' ? 'x86' : process.arch;
    return `node-${platform}-${arch}`;
}
async function readNodeVersionsManifest(nodesDir) {
    try {
        return await load_json_file_1.default(path_1.default.join(nodesDir, 'versions.json'));
    }
    catch (err) {
        if (err.code === 'ENOENT') {
            return {};
        }
        throw err;
    }
}
//# sourceMappingURL=node.js.map